package weka.filters.unsupervised.attribute;

import java.util.Enumeration;
import java.util.LinkedList;

/**
 * Splits the given string into an n-gram with min n_min and max n_max grams.
 * 
 * 
 * @author Sebastian Germesin (sebastian.germesin@dfki.de)
 *
 */

public class StringToNGramTokenizer implements Enumeration {
	
	private int NMAX;
	private int NMIN;
	private int N;
	private int MAXPOSITION;
	private int CURRENTPOSITION;
	private String[] SPLITSTRING;
	
	
	public StringToNGramTokenizer (String str, int n_max, int n_min, String delimiters) {
		if (n_max < 1) NMAX = 1;
		else NMAX = n_max;
		if (n_min < 1) NMIN = 1;
		else NMIN = n_min;
		N = NMAX;
		SPLITSTRING = str.split("[" + delimiters + "]");
		filterOutEmptyStrings();
		CURRENTPOSITION = 0;
		MAXPOSITION = SPLITSTRING.length;
	}
	
	
	public boolean hasMoreElements () {
		return (CURRENTPOSITION < MAXPOSITION && 
				N - 1 + CURRENTPOSITION < MAXPOSITION && 
				N >= NMIN);
	}
	/**
	 * Returns N-grams and also (N-1)-grams and .... and 1-grams.
	 */
	public Object nextElement () {
		String retValue = "";
		for (int i = 0; i < N && i + CURRENTPOSITION < MAXPOSITION; i++)
			retValue += " " + SPLITSTRING[CURRENTPOSITION + i];
		CURRENTPOSITION++;
		if (CURRENTPOSITION + N - 1 == MAXPOSITION) {
			CURRENTPOSITION = 0;
			N--;
		}
		
		return retValue.trim();
	}
	
	/** 
	 * filters out empty strings in SPLITSTRING and
	 * replaces SPLITSTRING with the cleaned version.
	 *
	 */
	private void filterOutEmptyStrings() {
		String[] newSplit;
		LinkedList<String> clean = new LinkedList<String>();
		
		for (int i = 0; i < SPLITSTRING.length; i++)
			if (!SPLITSTRING[i].equals("")) clean.add(SPLITSTRING[i]);
		
		newSplit = new String[clean.size()];
		for (int i = 0; i < clean.size(); i++) 
			newSplit[i] = clean.get(i);
		
		SPLITSTRING = newSplit;
	}
}
